"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
class LoggerClass {
    constructor() {
        this._warnings = [];
        this._silent = false;
    }
    setSilent(silent) {
        this._silent = silent;
    }
    info(message = '') {
        if (!this._silent) {
            console.log(chalk_1.default.cyan(`  ${message}`));
        }
    }
    error(message, exitProgram = false) {
        if (exitProgram) {
            commander_1.program.error(chalk_1.default.red(`  [error]  ${message}`));
        }
        else {
            if (!this._silent) {
                console.log(chalk_1.default.red(`  [error]  ${message}`));
            }
        }
    }
    warning(message) {
        this._warnings.push(message);
        if (!this._silent) {
            console.log(chalk_1.default.yellow(`  [warning]  ${message}`));
        }
    }
    created(message) {
        if (!this._silent) {
            console.log(chalk_1.default.green(`  [created]  ${message}`));
        }
    }
    resetWarnings() {
        this._warnings = [];
    }
    getWarnings() {
        return this._warnings;
    }
    getWarningCount() {
        return this._warnings.length;
    }
}
const Logger = new LoggerClass();
exports.default = Logger;
